import http.server
import io
import cgi
import logging

from remote_cmder.modules.cmder import Cmder

logger = logging.getLogger(__name__)


class CmderHTTPRequestHandler(http.server.SimpleHTTPRequestHandler):
    def __init__(self, *args, **kwargs):
        self.cmder = Cmder()
        super().__init__(*args, **kwargs)

    def do_POST(self):
        ret, info = self.post_cmder()
        logger.info(f"{ret} {info} by {self.client_address}: {self.path}")
        with io.BytesIO() as f:
            http_status = 400
            if ret:
                f.write(f"{info}\n".encode())
                http_status = 200
            else:
                f.write(f"Failed: {info}\n".encode())
                http_status = 400
            length = f.tell()
            f.seek(0)
            self.send_response(http_status)
            self.send_header("Content-type", "text/plain")
            self.send_header("Content-Length", str(length))
            self.end_headers()
            self.copyfile(f, self.wfile)

    def post_cmder(self):
        cmd = self.path[1:]
        if not self.cmder.is_cmd_supported(cmd):
            return (False, "Cmd doesn't supported")

        ctype, _ = cgi.parse_header(self.headers["Content-Type"])
        msg = ""
        if ctype == "multipart/form-data":
            form = cgi.FieldStorage(
                fp=self.rfile,
                headers=self.headers,
                environ={
                    "REQUEST_METHOD": "POST",
                    "CONTENT_TYPE": self.headers["Content-Type"],
                },
            )
            try:
                if isinstance(form["file"], list):
                    for record in form["file"]:
                        filename = record.name
                        file_content = record.file.read()
                        msg += self.__cmd_response(cmd, filename, file_content)
                else:
                    filename = form["file"].name
                    file_content = form["file"].file.read()
                    msg += self.__cmd_response(cmd, filename, file_content)
            except IOError:
                return (
                    False,
                    "Can't create file to write, do you have permission to write?",
                )
        return (True, msg)

    def __cmd_response(self, cmd, filename, file_content):
        msg = ""
        msg += f" {filename} ".center(40, "=") + "\n"
        ret, result = self.cmder.execute(cmd, filename, file_content)
        if ret:
            msg += result
        else:
            msg += "- cmder failed -"
        msg += "\n"
        return msg



from logging import config


def init_logger():
    log_config = {
        "version": 1,
        "root": {"handlers": ["console"], "level": "DEBUG"},
        "handlers": {
            "console": {
                "formatter": "std_out",
                "class": "logging.StreamHandler",
                "level": "DEBUG",
            }
        },
        "formatters": {
            "std_out": {
                "format": "%(asctime)s | %(levelname)s | %(module)s::%(funcName)s:%(lineno)d | (Process Details: (%(process)d, %(processName)s), Thread Details: (%(thread)d, %(threadName)s)) | %(message)s",
                "datefmt": "%Y-%m-%d %H:%M:%S",
            }
        },
    }
    config.dictConfig(log_config)



import hashlib


class Cmder:
    def __init__(self):
        self.__cmd_map = {
            "md5": self.__cmd_md5,
            "sha1": self.__cmd_sha1,
        }

    def execute(self, cmd, filename, data, *args, **kwargs):
        return self.__cmd_map[cmd](filename, data, *args, **kwargs)

    def is_cmd_supported(self, cmd):
        return cmd in self.__cmd_map

    def __cmd_md5(self, filename, data, *args, **kwargs):
        m = hashlib.md5()
        m.update(data)
        h = m.hexdigest()
        return (
            True,
            f"{filename}: {h}",
        )

    def __cmd_sha1(self, filename, data, *args, **kwargs):
        m = hashlib.sha1()
        m.update(data)
        h = m.hexdigest()
        return (
            True,
            f"{filename}: {h}",
        )



import argparse
import logging
import socketserver

from .settings import init_settings
from .modules.server import CmderHTTPRequestHandler

init_settings()
logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("--port", type=int, default=8888)

    args = parser.parse_args()
    port = args.port

    Handler = CmderHTTPRequestHandler
    with socketserver.TCPServer(("", port), Handler) as httpd:
        logger.info(f"serving at port: {port}")
        httpd.serve_forever()


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

requirements = []
with open("requirements.txt", "r") as fh:
    for line in fh:
        requirements.append(line.strip())

setuptools.setup(
    name="remote-cmder",
    version="0.0.3",
    author="Zonda Yang",
    author_email="u226699@gmail.com",
    description="Remote cmder",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zondatw/remote_cmder",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    keywords="remote cmder",
    packages=["remote_cmder", "remote_cmder.modules", "remote_cmder.settings"],
    python_requires=">=3.7",
    entry_points={
        "console_scripts": [
            "remote-cmder=remote_cmder.cli:main",
        ],
    },
    install_requires=requirements,
)
